@@ -10,6 +10,9 @@
 #include <stddef.h>
 #include <assert.h>
 #include <errno.h>
+#if HAVE_HEADER_INTTYPES_H
+#include <inttypes.h>
+#endif
 
 // TODO: fix this to make it crosscompile-friendly: RZ_SYS_OSTYPE ?
 /* operating system */
@@ -18,6 +21,7 @@
 #undef __UNIX__
 #undef __WINDOWS__
 
+// TODO: these modes should be dropped when oldshell is removed in favour of RzOutputMode.
 #define RZ_MODE_PRINT     0x000
 #define RZ_MODE_RIZINCMD  0x001
 #define RZ_MODE_SET       0x002
@@ -43,14 +47,25 @@ typedef enum {
 	RZ_OUTPUT_MODE_QUIETEST = 1 << 8,
 } RzOutputMode;
 
-#define RZ_IN        /* do not use, implicit */
-#define RZ_OUT       /* parameter is written, not read */
-#define RZ_INOUT     /* parameter is read and written */
+#define RZ_IN    /* do not use, implicit */
+#define RZ_OUT   /* parameter is written, not read */
+#define RZ_INOUT /* parameter is read and written */
+
+#ifdef RZ_BINDINGS
+#define RZ_OWN    __attribute__((annotate("RZ_OWN")))
+#define RZ_BORROW __attribute__((annotate("RZ_BORROW")))
+
+#define RZ_NONNULL   __attribute__((annotate("RZ_NONNULL")))
+#define RZ_NULLABLE  __attribute__((annotate("RZ_NULLABLE")))
+#define RZ_DEPRECATE __attribute__((annotate("RZ_DEPRECATE")))
+#else
 #define RZ_OWN       /* pointer ownership is transferred */
 #define RZ_BORROW    /* pointer ownership is not transferred, it must not be freed by the receiver */
 #define RZ_NONNULL   /* pointer can not be null */
 #define RZ_NULLABLE  /* pointer can be null */
 #define RZ_DEPRECATE /* should not be used in new code and should/will be removed in the future */
+#endif
+
 #define RZ_IFNULL(x) /* default value for the pointer when null */
 #ifdef __GNUC__
 #define RZ_UNUSED __attribute__((__unused__))
@@ -136,33 +151,11 @@ typedef enum {
 #define __UNIX__ 1
 #endif
 #if __WINDOWS__ || _WIN32
-#ifdef _MSC_VER
-#include <sdkddkver.h>
-#ifdef NTDDI_WIN10_TH2
-/* Avoid using Developer Preview and default to Windows 10/Windows Server 2016 */
-#undef _WIN32_WINNT
-#undef NTDDI_VERSION
-#define _WIN32_WINNT  _WIN32_WINNT_WIN10
-#define NTDDI_VERSION NTDDI_WIN10
-#endif
-/* Must be included before windows.h */
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#ifndef WIN32_LEAN_AND_MEAN
-#define WIN32_LEAN_AND_MEAN
-#endif
-#endif
-typedef int socklen_t;
-#undef USE_SOCKETS
-#define __WINDOWS__ 1
+#define __WINDOWS__  1
+#define _WINSOCKAPI_ /* Prevent inclusion of winsock.h in windows.h */
 #undef __UNIX__
 #undef __BSD__
 #endif
-#if __WINDOWS__ || _WIN32
-#define __addr_t_defined
-#include <windows.h>
-#include <direct.h>
-#endif
 
 #if defined(__APPLE__) && ((__arm__ || __arm64__ || __aarch64__) && IS_IOS)
 #define TARGET_OS_IPHONE 1
@@ -200,9 +193,8 @@ typedef int socklen_t;
 #endif
 
 #include <rz_types_base.h>
+#include <rz_constructor.h>
 
-#undef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS 64
 #undef _GNU_SOURCE
 #define _GNU_SOURCE
 
@@ -217,16 +209,12 @@ typedef int socklen_t;
 extern "C" {
 #endif
 
-// TODO: FS or RZ_SYS_DIR ??
-#undef FS
 #if __WINDOWS__
-#define FS            "\\"
 #define RZ_SYS_DIR    "\\"
 #define RZ_SYS_ENVSEP ";"
 #define RZ_SYS_HOME   "USERPROFILE"
 #define RZ_SYS_TMP    "TEMP"
 #else
-#define FS            "/"
 #define RZ_SYS_DIR    "/"
 #define RZ_SYS_ENVSEP ":"
 #define RZ_SYS_HOME   "HOME"
@@ -268,8 +256,8 @@ typedef int (*PrintfCallback)(const char *str, ...) RZ_PRINTF_CHECK(1, 2);
 #ifdef RZ_API
 #undef RZ_API
 #endif
-#if RZ_SWIG
-#define RZ_API export
+#ifdef RZ_BINDINGS
+#define RZ_API __attribute__((annotate("RZ_API")))
 #elif RZ_INLINE
 #define RZ_API inline
 #else
@@ -285,7 +273,9 @@ typedef int (*PrintfCallback)(const char *str, ...) RZ_PRINTF_CHECK(1, 2);
 #define RZ_LIB_VERSION_HEADER(x) \
 	RZ_API const char *x##_version(void)
 #define RZ_LIB_VERSION(x) \
-	RZ_API const char *x##_version(void) { return "" RZ_VERSION; }
+	RZ_API const char *x##_version(void) { \
+		return "" RZ_VERSION; \
+	}
 
 #define BITS2BYTES(x)    (((x) / 8) + (((x) % 8) ? 1 : 0))
 #define ZERO_FILL(x)     memset(&x, 0, sizeof(x))
@@ -317,11 +307,12 @@ static inline void *rz_new_copy(int size, void *data) {
 #define RZ_PTR_ALIGN_NEXT(v, t) \
 	((char *)(((size_t)(v) + (t - 1)) & ~(t - 1)))
 
+#define RZ_BIT_MASK32(x, y) ((1UL << (x)) - (1UL << (y)))
 #define RZ_BIT_SET(x, y)    (((ut8 *)x)[y >> 4] |= (1 << (y & 0xf)))
 #define RZ_BIT_UNSET(x, y)  (((ut8 *)x)[y >> 4] &= ~(1 << (y & 0xf)))
 #define RZ_BIT_TOGGLE(x, y) (RZ_BIT_CHK(x, y) ? RZ_BIT_UNSET(x, y) : RZ_BIT_SET(x, y))
 
-//#define RZ_BIT_CHK(x,y) ((((const ut8*)x)[y>>4] & (1<<(y&0xf))))
+// #define RZ_BIT_CHK(x,y) ((((const ut8*)x)[y>>4] & (1<<(y&0xf))))
 #define RZ_BIT_CHK(x, y) (*(x) & (1 << (y)))
 
 /* try for C99, but provide backwards compatibility */
@@ -365,10 +356,10 @@ static inline void *rz_new_copy(int size, void *data) {
 
 // There is a bug of using "offsetof()" in the structure
 // initialization in GCC < 5.0 versions
-#if !(defined(__GNUC__) && __GNUC__ < 5)
+#if !(defined(__GNUC__) && __GNUC__ < 5) || defined(__clang__)
 #define rz_offsetof(type, member) offsetof(type, member)
 #else
-#if __SDB_WINDOWS__
+#if __WINDOWS__
 #define rz_offsetof(type, member) ((unsigned long)(ut64) & ((type *)0)->member)
 #else
 #define rz_offsetof(type, member) ((unsigned long)&((type *)0)->member)
@@ -381,28 +372,36 @@ static inline void *rz_new_copy(int size, void *data) {
 		x = NULL; \
 	}
 
+#define RZ_FREE_CUSTOM(x, y) \
+	{ \
+		y(x); \
+		x = NULL; \
+	}
+
 #if __WINDOWS__
-#define PFMT64x "I64x"
-#define PFMT64d "I64d"
-#define PFMT64u "I64u"
-#define PFMT64o "I64o"
-#define PFMTSZx "Ix"
-#define PFMTSZd "Id"
-#define PFMTSZu "Iu"
-#define PFMTSZo "Io"
-#define LDBLFMT "f"
-#define HHXFMT  "x"
+#define PFMT64x  "I64x"
+#define PFMT64d  "I64d"
+#define PFMT64u  "I64u"
+#define PFMT64o  "I64o"
+#define PFMTSZx  "Ix"
+#define PFMTSZd  "Id"
+#define PFMTSZu  "Iu"
+#define PFMTSZo  "Io"
+#define LDBLFMTg "g"
+#define LDBLFMTf "f"
+#define HHXFMT   "x"
 #else
-#define PFMT64x "llx"
-#define PFMT64d "lld"
-#define PFMT64u "llu"
-#define PFMT64o "llo"
-#define PFMTSZx "zx"
-#define PFMTSZd "zd"
-#define PFMTSZu "zu"
-#define PFMTSZo "zo"
-#define LDBLFMT "Lf"
-#define HHXFMT  "hhx"
+#define PFMT64x  "llx"
+#define PFMT64d  "lld"
+#define PFMT64u  "llu"
+#define PFMT64o  "llo"
+#define PFMTSZx  "zx"
+#define PFMTSZd  "zd"
+#define PFMTSZu  "zu"
+#define PFMTSZo  "zo"
+#define LDBLFMTg "Lg"
+#define LDBLFMTf "Lf"
+#define HHXFMT   "hhx"
 #endif
 
 #define PFMTDPTR "td"
@@ -497,16 +496,26 @@ static inline void *rz_new_copy(int size, void *data) {
 #endif
 #else
 #ifdef _MSC_VER
-#ifdef _WIN64
+#if defined(_M_X64) || defined(_M_AMD64)
 #define RZ_SYS_ARCH   "x86"
 #define RZ_SYS_BITS   (RZ_SYS_BITS_32 | RZ_SYS_BITS_64)
 #define RZ_SYS_ENDIAN 0
 #define __x86_64__    1
-#else
+#elif defined(_M_IX86)
 #define RZ_SYS_ARCH   "x86"
 #define RZ_SYS_BITS   (RZ_SYS_BITS_32)
-#define __i386__      1
 #define RZ_SYS_ENDIAN 0
+#define __i386__      1
+#elif defined(_M_ARM64)
+#define RZ_SYS_ARCH   "arm"
+#define RZ_SYS_BITS   (RZ_SYS_BITS_32 | RZ_SYS_BITS_64)
+#define RZ_SYS_ENDIAN 0
+#define __arm64__     1
+#elif defined(_M_ARM)
+#define RZ_SYS_ARCH   "arm"
+#define RZ_SYS_BITS   RZ_SYS_BITS_32
+#define RZ_SYS_ENDIAN 0
+#define __arm__       1
 #endif
 #else
 #define RZ_SYS_ARCH   "unknown"
@@ -560,8 +569,8 @@ typedef enum {
 /* os */
 #if defined(__QNX__)
 #define RZ_SYS_OS "qnx"
-//#elif TARGET_OS_IPHONE
-//#define RZ_SYS_OS "ios"
+// #elif TARGET_OS_IPHONE
+// #define RZ_SYS_OS "ios"
 #elif defined(__APPLE__)
 #define RZ_SYS_OS "darwin"
 #elif defined(__linux__)
@@ -660,7 +669,13 @@ typedef int RzRef;
 
 #define RZ_REF_TYPE RzRef RZ_REF_NAME
 #define RZ_REF_FUNCTIONS(s, n) \
-	static inline void n##_ref(s *x) { x->RZ_REF_NAME++; } \
-	static inline void n##_unref(s *x) { rz_unref(x, n##_free); }
+	static inline void n##_ref(s *x) { \
+		x->RZ_REF_NAME++; \
+	} \
+	static inline void n##_unref(s *x) { \
+		rz_unref(x, n##_free); \
+	}
+
+typedef struct rz_core_t RzCore;
 
 #endif // RZ_TYPES_H
